import { Canvas, CanvasView, Paint } from '@nativescript-community/ui-canvas';
import { booleanConverter, Property } from '@nativescript/core';
import { HTMLCanvasElement } from '../elements/HTMLCanvasElement';
import { SCREEN_SCALE } from '../helpers';
function onLayoutChange(args) {
    const view = args.object;
    // During view layout, offscreen buffer is also resized
    if (view.isOffscreenBufferEnabled) {
        const { width, height } = view.getActualSize();
        if (width != 0 && height != 0) {
            view._resizeOffscreenBuffer(width, height);
        }
        else {
            view._disposeOffscreenBuffer();
        }
    }
}
const isOffscreenBufferEnabledProperty = new Property({
    name: 'isOffscreenBufferEnabled',
    defaultValue: false,
    valueConverter: booleanConverter,
});
class HTMLCanvasViewBase extends CanvasView {
    constructor() {
        super();
        this._canvasElement = new HTMLCanvasElement(this);
    }
    get canvasElement() {
        return this._canvasElement;
    }
    get nativeContext() {
        return this._offscreenContext != null ? this._offscreenContext : this._currentCanvas;
    }
    disposeNativeView() {
        super.disposeNativeView();
        this._disposeOffscreenBuffer();
    }
    getContext(contextId, contextAttributes) {
        return this._canvasElement.getContext(contextId, contextAttributes);
    }
    transferControlToOffscreen() {
        return this._canvasElement.transferControlToOffscreen();
    }
    toDataURL(type, encoderOptions) {
        return this._canvasElement.toDataURL(type, encoderOptions);
    }
    isDrawing() {
        return this._isDrawing;
    }
    onDraw(canvas) {
        this._isDrawing = true;
        this._currentCanvas = canvas;
        super.onDraw(canvas);
        if (this._offscreenContext != null) {
            canvas.drawBitmap(this._offscreenContext.getImage(), 0, 0, this._offscreenPaint);
        }
        this._currentCanvas = null;
        this._isDrawing = false;
    }
    _resizeOffscreenBuffer(width, height) {
        if (this._offscreenContext != null) {
            this._offscreenContext.release();
        }
        this._offscreenContext = new Canvas(width, height);
    }
    _disposeOffscreenBuffer() {
        if (this._offscreenContext != null) {
            this._offscreenContext.release();
            this._offscreenContext = null;
        }
        this._offscreenPaint = null;
    }
    [isOffscreenBufferEnabledProperty.setNative](value) {
        if (value) {
            this._offscreenPaint = new Paint();
            this._offscreenPaint.setAntiAlias(true);
            this.on(HTMLCanvasViewBase.layoutChangedEvent, onLayoutChange);
        }
        else {
            this.off(HTMLCanvasViewBase.layoutChangedEvent, onLayoutChange);
            this._disposeOffscreenBuffer();
        }
    }
}
isOffscreenBufferEnabledProperty.register(HTMLCanvasViewBase);
export { SCREEN_SCALE, HTMLCanvasViewBase, isOffscreenBufferEnabledProperty };
//# sourceMappingURL=html-canvas-view-common.js.map